/* Copyright (C) 2008, Cyril Hu (cyrilhu@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

#if defined(STR_DEBUG) || defined(STR_STRRSTR_DEBUG)
#include <stdio.h>
#endif
#include <stddef.h>
#include <string.h>

/* Reversed strstr(), return a pointer to the beginning of 
 * the last matching sub, or NULL if not found. */
char *strrstr(const char *src, const char *sub)
{
	size_t ls = strlen(sub);
	char *p = NULL, *t;

#ifdef STR_DEBUG
	printf("\nstrrstr(): src=\"%s\", sub=\"%s\"\n", src, sub);
#endif
	if(!ls) {
#ifdef STR_DEBUG
		printf("strrstr(): sanity check failed, strlen(sub)=%u, returning \"%s\"\n", (unsigned)ls, src);
#endif
		return (char *)src;
	}
	while(*src) {
		t = strstr(src, sub);
		if(t) {
			p = t, src = t + ls;
		} else 
			break;
	}
#ifdef STR_DEBUG
	printf("strrstr(): done, returning \"%s\"\n", p);
#endif
	return p;
}

#ifdef STR_STRRSTR_DEBUG
int main(void)
{
	strrstr("", "");
	strrstr("lasdjflkasjf", "");
	strrstr("", "asd");
	strrstr("lasdjflkasjf", "as");
	strrstr("lasdjflkasjf", "y");
	strrstr("lasdjflkasjf", "ly");
	strrstr("lasdjflkasjf", "j");
	strrstr("lasdjflkasjf", "f");
	strrstr("lasdjflkasjf", "d");
	strrstr("lasdjflkasjf", "lasdjflkasjf");
	strrstr("lasdjflkasjf", "lasdjflkasjfaskldfj");
	return 0;
}
#endif
